@@ -15,24 +15,22 @@
 #
 # END HEADER
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import OrderedDict, hbytes
-from hypothesis.searchstrategy.strategies import SearchStrategy, \
-    MappedSearchStrategy, combine_labels, one_of_strategies
+from hypothesis.internal.compat import OrderedDict
+from hypothesis.internal.conjecture.utils import combine_labels
+from hypothesis.searchstrategy.strategies import MappedSearchStrategy, SearchStrategy
 
 
 class TupleStrategy(SearchStrategy):
     """A strategy responsible for fixed length tuples based on heterogenous
     strategies for each of their elements."""
 
-    def __init__(self,
-                 strategies, tuple_type):
+    def __init__(self, strategies):
         SearchStrategy.__init__(self)
-        strategies = tuple(strategies)
-        self.element_strategies = strategies
+        self.element_strategies = tuple(strategies)
 
     def do_validate(self):
         for s in self.element_strategies:
@@ -40,59 +38,40 @@ class TupleStrategy(SearchStrategy):
 
     def calc_label(self):
         return combine_labels(
-            self.class_label, *[s.label for s in self.element_strategies])
+            self.class_label, *[s.label for s in self.element_strategies]
+        )
 
     def __repr__(self):
         if len(self.element_strategies) == 1:
-            tuple_string = '%s,' % (repr(self.element_strategies[0]),)
+            tuple_string = "%s," % (repr(self.element_strategies[0]),)
         else:
-            tuple_string = ', '.join(map(repr, self.element_strategies))
-        return 'TupleStrategy((%s))' % (
-            tuple_string,
-        )
+            tuple_string = ", ".join(map(repr, self.element_strategies))
+        return "TupleStrategy((%s))" % (tuple_string,)
 
     def calc_has_reusable_values(self, recur):
         return all(recur(e) for e in self.element_strategies)
 
-    def newtuple(self, xs):
-        """Produce a new tuple of the correct type."""
-        return tuple(xs)
-
     def do_draw(self, data):
-        return self.newtuple(
-            data.draw(e) for e in self.element_strategies
-        )
+        return tuple(data.draw(e) for e in self.element_strategies)
 
     def calc_is_empty(self, recur):
         return any(recur(e) for e in self.element_strategies)
 
 
-TERMINATOR = hbytes(b'\0')
-
-
 class ListStrategy(SearchStrategy):
-    """A strategy for lists which takes an intended average length and a
-    strategy for each of its element types and generates lists containing any
-    of those element types.
+    """A strategy for lists which takes a strategy for its elements and the
+    allowed lengths, and generates lists with the correct size and contents."""
 
-    The conditional distribution of the length is geometric, and the
-    conditional distribution of each parameter is whatever their
-    strategies define.
-
-    """
-
-    def __init__(
-        self,
-        strategies, average_length=50.0, min_size=0, max_size=float('inf')
-    ):
+    def __init__(self, elements, min_size=0, max_size=float("inf")):
         SearchStrategy.__init__(self)
-
-        assert average_length > 0
-        self.average_length = average_length
-        strategies = tuple(strategies)
         self.min_size = min_size or 0
-        self.max_size = max_size or float('inf')
-        self.element_strategy = one_of_strategies(strategies)
+        self.max_size = max_size if max_size is not None else float("inf")
+        assert 0 <= self.min_size <= self.max_size
+        self.average_size = min(
+            max(self.min_size * 2, self.min_size + 5),
+            0.5 * (self.min_size + self.max_size),
+        )
+        self.element_strategy = elements
 
     def calc_label(self):
         return combine_labels(self.class_label, self.element_strategy.label)
@@ -100,10 +79,22 @@ class ListStrategy(SearchStrategy):
     def do_validate(self):
         self.element_strategy.validate()
         if self.is_empty:
-            raise InvalidArgument((
-                'Cannot create non-empty lists with elements drawn from '
-                'strategy %r because it has no values.') % (
-                self.element_strategy,))
+            raise InvalidArgument(
+                (
+                    "Cannot create non-empty lists with elements drawn from "
+                    "strategy %r because it has no values."
+                )
+                % (self.element_strategy,)
+            )
+        if self.element_strategy.is_empty and 0 < self.max_size < float("inf"):
+            from hypothesis._settings import note_deprecation
+
+            note_deprecation(
+                "Cannot create a collection of max_size=%r, because no "
+                "elements can be drawn from the element strategy %r"
+                % (self.max_size, self.element_strategy),
+                since="2018-03-11",
+            )
 
     def calc_is_empty(self, recur):
         if self.min_size == 0:
@@ -118,8 +109,9 @@ class ListStrategy(SearchStrategy):
 
         elements = cu.many(
             data,
-            min_size=self.min_size, max_size=self.max_size,
-            average_size=self.average_length
+            min_size=self.min_size,
+            max_size=self.max_size,
+            average_size=self.average_size,
         )
         result = []
         while elements.more():
@@ -127,43 +119,19 @@ class ListStrategy(SearchStrategy):
         return result
 
     def __repr__(self):
-        return (
-            'ListStrategy(%r, min_size=%r, average_size=%r, max_size=%r)'
-        ) % (
-            self.element_strategy, self.min_size, self.average_length,
-            self.max_size
+        return "%s(%r, min_size=%r, max_size=%r)" % (
+            self.__class__.__name__,
+            self.element_strategy,
+            self.min_size,
+            self.max_size,
         )
 
 
-class UniqueListStrategy(SearchStrategy):
-
-    def __init__(
-        self,
-        elements, min_size, max_size, average_size,
-        key
-    ):
-        super(UniqueListStrategy, self).__init__()
-        assert min_size <= average_size <= max_size
-        self.min_size = min_size
-        self.max_size = max_size
-        self.average_size = average_size
-        self.element_strategy = elements
+class UniqueListStrategy(ListStrategy):
+    def __init__(self, elements, min_size, max_size, key):
+        super(UniqueListStrategy, self).__init__(elements, min_size, max_size)
         self.key = key
 
-    def do_validate(self):
-        self.element_strategy.validate()
-        if self.is_empty:
-            raise InvalidArgument((
-                'Cannot create non-empty lists with elements drawn from '
-                'strategy %r because it has no values.') % (
-                self.element_strategy,))
-
-    def calc_is_empty(self, recur):
-        if self.min_size == 0:
-            return False
-        else:
-            return recur(self.element_strategy)
-
     def do_draw(self, data):
         if self.element_strategy.is_empty:
             assert self.min_size == 0
@@ -171,8 +139,9 @@ class UniqueListStrategy(SearchStrategy):
 
         elements = cu.many(
             data,
-            min_size=self.min_size, max_size=self.max_size,
-            average_size=self.average_size
+            min_size=self.min_size,
+            max_size=self.max_size,
+            average_size=self.average_size,
         )
         seen = set()
         result = []
@@ -195,7 +164,6 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
 
     e.g. {'foo' : some_int_strategy} would generate dicts with the single
     key 'foo' mapping to some integer.
-
     """
 
     def __init__(self, strategy_dict):
@@ -205,25 +173,18 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
             self.keys = tuple(strategy_dict.keys())
         else:
             try:
-                self.keys = tuple(sorted(
-                    strategy_dict.keys(),
-                ))
+                self.keys = tuple(sorted(strategy_dict.keys()))
             except TypeError:
-                self.keys = tuple(sorted(
-                    strategy_dict.keys(), key=repr,
-                ))
+                self.keys = tuple(sorted(strategy_dict.keys(), key=repr))
         super(FixedKeysDictStrategy, self).__init__(
-            strategy=TupleStrategy(
-                (strategy_dict[k] for k in self.keys), tuple
-            )
+            strategy=TupleStrategy((strategy_dict[k] for k in self.keys))
         )
 
     def calc_is_empty(self, recur):
         return recur(self.mapped_strategy)
 
     def __repr__(self):
-        return 'FixedKeysDictStrategy(%r, %r)' % (
-            self.keys, self.mapped_strategy)
+        return "FixedKeysDictStrategy(%r, %r)" % (self.keys, self.mapped_strategy)
 
     def pack(self, value):
         return self.dict_type(zip(self.keys, value))
